#!/srv/epages/eproot/Perl/bin/perl

#========================================================================================
# SCRIPT WHICH TURNS HTML FILES INTO CTMPL FILES
#========================================================================================

use strict;
use Getopt::Long;
use Cwd;
use File::Find qw(find);
use Time::HiRes qw (time);

use DE_EPAGES::Core::API::File qw ( WriteFile );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Dictionary::API::Template;

sub Main {

    local $| = 1;

    my $time1 = time;

    #-------------------------------------------------------------------
    # check options
    #-------------------------------------------------------------------

    my %Options = ();
    # set defaults
    $Options{outdir} = '.';

    my @GetOpt = qw(help cartridge=s lang=s outdir=s nooutput);
    GetOptions(\%Options, @GetOpt);

    usage() if $Options{help};
    foreach (qw(lang cartridge)) {
        usage("Option -$_ missing") unless defined $Options{$_};
    }

    # File::Find has problems with '.'
    $Options{outdir} = getcwd() if $Options{outdir} eq '.';

    my $CartridgeDir = $Options{cartridge};
    $CartridgeDir =~ s,::,/,g;
    usage('Option -cartridge must not be absolute path') if $CartridgeDir =~ m,^/,;
    usage("No such directory: $ENV{EPAGES_CARTRIDGES}/$CartridgeDir")
        unless -d "$ENV{EPAGES_CARTRIDGES}/$CartridgeDir";

    #-------------------------------------------------------------------
    # create ctmpl
    #-------------------------------------------------------------------

    my $Handler = DE_EPAGES::Dictionary::API::Template->new( force => 1 );

    find(sub
         {
             next unless -f $_;
             next if /\.xml$/;
             next if /\.xsl$/;
             my $TargetFile = $File::Find::name;
             $TargetFile =~ s,.*/$CartridgeDir/Templates/,,;
             $TargetFile =~ s,\.[^.]+$,,;
             foreach my $Lang (split(/\s+/, $Options{lang})) {
                 my $CtmplFile = "$Options{outdir}/$Lang/$CartridgeDir/$TargetFile.$Lang.ctmpl";
                 print "$CtmplFile\n" unless $Options{nooutput};
                 my $rPerlCode = $Handler->GetPerlCodeByTemplate($File::Find::name, $Lang);
                 $$rPerlCode =~ s,"YOUR_TEMPLATE","$File::Find::name",g;
                 WriteFile($CtmplFile, $rPerlCode, { 'create_directory'=> 1, 'utf8' => 1,
                                                     'flock' => 1, 'ignore_error'=>1 });
             }
         },
         "$ENV{EPAGES_CARTRIDGES}/$CartridgeDir/Templates"
        );

    printf "need %.1f seconds%30s\n", time-$time1, '' unless $Options{nooutput};
}

sub usage {
    my ($Msg) = @_;
    print "ERROR: $Msg\n\n" if $Msg;
    print <<END_USAGE;
Description:
    Compiles all CARTRIDGE/Templates/**.html to OUTDIR/LANG/**.LANG.ctmpl

Usage:
    perl $0 [options] [flags]

Options:
    -cartridge cartridge name
    -outdir    target directory for ctmpl files (default: .)
    -lang      create ctmpls for those languages

Flags:
    -help       show the command line options
    -nooutput   do not show progress

Examples:
    perl $0 -cartridge DE_EPAGES/Shop -outdir /tmp/Templates -lang de
    perl $0 -cartridge DE_EPAGES::Shop -lang 'de en'

END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
    Debug => 1
);
