#========================================================================================
# package      DE_EPAGES::ExternalShipping::Hooks::ShippingLabel
# state        public
#----------------------------------------------------------------------------------------
# description  ShippingLabel hook listener
#========================================================================================
package DE_EPAGES::ExternalShipping::Hooks::ShippingLabel;

use strict;

use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime isValid );
#========================================================================================
# function     OnUpdateShippingLabel
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnUpdateShippingLabel( $hInfo );
#----------------------------------------------------------------------------------------
# description  OBJ_UpdateShippingLabel hook listener. updates ProductCode to ShippingLabel
#               if a product was chosen.
#----------------------------------------------------------------------------------------
# input        $hInfo | Hook parameters. Used keys:<ul>
#                 <li>Object - ShippingLabel</li>
#                 <li>Values - updated values</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnUpdateShippingLabel {
    my ($hInfo) = @_;
    my $Object = $hInfo->{'Object'};
    TestObject('Object',$Object,'DE_EPAGES::ExternalShipping::API::Object::ShippingLabel');
    my $hValues = $hInfo->{'Values'};
    # has ShippingFulfillmentProduct been selected?
    if (exists $hValues->{'ShippingFulfillmentProduct'}) {
        my $Product = $hValues->{'ShippingFulfillmentProduct'};
        if (!defined $Product) {
            $Object->set({'ProductCode' => undef});
            return;
        } else {
            $Object->set({'ProductCode'=> $Product->get('ProductCode')});
        }
    }

    if (isValid($hValues->{'TrackingID'})) {
        my $hUpdateInfo = {
            'Issued' => 1,
        };
        my $TrackingLink = $Object->generateTrackingLink();
        $hUpdateInfo->{'TrackingLink'} = $TrackingLink if ($TrackingLink);
        $Object->set($hUpdateInfo);
        my $Order = $Object->get('Order');
        $Order->set({'TrackingURL' => $TrackingLink}) if ($TrackingLink);
        $Order->set({'DispatchedOn' => GetCurrentDateTime}) if(defined($Order) && !defined($Order->get('DispatchedOn')));
    }
}

1;