#========================================================================================
# package      DE_EPAGES::OpenAI::API::TLE::OpenAIHandler
# state        public
#----------------------------------------------------------------------------------------
# description 
#========================================================================================
package DE_EPAGES::OpenAI::API::TLE::OpenAIHandler;

use strict;
use DE_EPAGES::OpenAI::API::Tools qw ( GetRemainingTokens );

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::OpenAI::API::TLE::OpenAIHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift;}

#========================================================================================
# function     tle
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = $Handler->tle( $Processor, $TLEName);
# example      my $Value = $Handler->tle( $Processor, 'Name');
#----------------------------------------------------------------------------------------
# description  This function is called by tle processor to get the value of a tle
#               variable.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name | string
# return       $Value | return value | integer
#========================================================================================
sub tle {
    my $self = shift;
    my ($Processor, $TLEName) = @_;

    my $Object = $Processor->tle('Object');
    my $Shop = $Object->getSite();

    if( $TLEName eq 'OpenAIRemainingTokens' ) {
        return GetRemainingTokens($Shop);
    }

    return undef;
}

#========================================================================================
# function     existsTLE
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = $Handler->existsTLE( $Processor, $TLEName);
# example      my $Value = $Handler->existsTLE( $Processor, 'Products_Count');
#----------------------------------------------------------------------------------------
# description  This function is called by the tle processor to test if a TLE variable
#               is available.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name | string
# return       $Value | yes or no | bool
#========================================================================================
sub existsTLE {
    my $self = shift;
    my ($Processor, $TLEName) = @_;

    return (defined $self->tle($Processor, $TLEName)) ? 1 : 0;
}

sub register {
    my $self = shift;
    my ($Processor) = @_;

    $Processor->registerHandler( 'VariableHandler', $self, 'OpenAIRemainingTokens');

    return;
}

1;

