#========================================================================================
# package      DE_EPAGES::Order::API::Attributes::LineItemShipping_Ext
# state        private
# description  Attribute interface for LineItemShipping.
#========================================================================================
package DE_EPAGES::Order::API::Attributes::LineItemShipping_Ext;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use DE_EPAGES::Core::API::Constants qw( PERL538 );


sub getAttributes {
    my $self = shift;
    my ($Object, $aNames) = @_;
    my $hInfo = {};

    foreach my $Name (@$aNames) {
        if ($Name eq 'Taxes') {
            $hInfo->{$Name} = $self->_shippingTaxes($Object);
        }
    }
    return $hInfo;
}


# get the shipping prices in a loop for each tax rate
# e.g. if we have 5,- shipping and products with 7% and 19% tax,
# shipping is splitted to the relation of the 7% and 19% produts of th total price
# from each shipping price the tax is calculated
sub _shippingTaxes {
    my $self = shift;
    my ($LineItemShipping) = @_;
    my $Deprecated = $] < PERL538 ? 1 : 0;
    $Deprecated = 0 if $Deprecated and $ENV{USE_NEW_LINEITEMCONTAINER};

    # get the TaxLineItems from Container
    my $Container = $LineItemShipping->container;
    my $aTaxLineItems = $Container->get('Taxes');
    my $NoTax = $Container->get('NoTax');

    # if the customer has removed a TaxClass, we do not calculate the splitten tax amounts
    return [] if $Container->get('IsTaxClassRemoved');

    my $hTaxRates = $Container->netPricesForTax();
    my $hShippingTaxItems = $hTaxRates->{'LineItemShipping'};
    my $Formatter = $Container->formatter;

    my $aRet;
    foreach my $TaxLineItem (@$aTaxLineItems) {

        my $TaxArea = $TaxLineItem->get('TaxArea');
        my $TaxClass = $TaxLineItem->get('TaxClass');
        return [] unless defined $TaxClass;

        my $TaxRate = defined $TaxArea
        ? ($NoTax ? 0 : $Container->taxRate($TaxClass))
        : $TaxLineItem->get('TaxRate');

        my $Value;
        if ($Deprecated) {
            $Value = $hShippingTaxItems->{$TaxClass->id} // 0;
        } else {
            my @TaxItem = grep { $_->[0] eq $TaxClass->id } @$hShippingTaxItems;
            $Value = @TaxItem ? $TaxItem[0][1] : 0;
        }
        next if ($Value eq '0') || !(defined $Value);

        push @$aRet, {'TaxClass'  => $TaxClass,
                      'TaxRate'   => $TaxRate,
                      'Value'     => $Formatter->round($Value),
                      'TaxAmount' => $TaxLineItem->calculateTaxAmount($Value, $TaxRate, $TaxRate),
        }
    }
    return $aRet;
}

1;
