#==============================================================================
# package      DE_EPAGES::PayPalPPCP::Hooks::Attribute
# state        private
#==============================================================================
package DE_EPAGES::PayPalPPCP::Hooks::Attribute;

use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::ThirdPartyConfig::API::Constants qw ( ATTRIBUTES_CATEGORY_PAYMENT );

use strict;

#========================================================================================
# function     OnGetThirdPartyConfigAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnGetThirdPartyConfigAttributes($hParams);
#----------------------------------------------------------------------------------------
# description  API_GetThirdPartyConfigAttributes triggered, register attributes to be
#               configured in PBO
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>AttributesCategory
#                   <li>LanguageID
#               </ul> | ref.hash.*
#========================================================================================
sub OnGetThirdPartyConfigAttributes {
    my ($hParams) = @_;

    return unless $hParams->{'AttributesCategory'} eq ATTRIBUTES_CATEGORY_PAYMENT;

    my $PaymentType  = LoadObjectByPath('/PaymentTypes/PayPalPPCP');

    push(@{$hParams->{'Attributes'}}, {
        'SectionID'   => $PaymentType->alias(),
        'SectionName' => $PaymentType->get('NameOrAlias', $hParams->{'LanguageID'}),
        'Attributes' => [
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'Logging'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'BaseURL'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerMerchantID'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerBNCodeECM'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerBNCodeECS'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerBNCodeECM_Base'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerBNCodeECS_Base'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerClientID'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerClientSecret'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'BaseURL_SandBox'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerMerchantID_SandBox'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerBNCodeECM_SandBox'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerBNCodeECS_SandBox'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerBNCodeECM_Base_SandBox'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerBNCodeECS_Base_SandBox'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerClientID_SandBox'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PartnerClientSecret_SandBox'},
        ]
    });

    return;
}

1;