#========================================================================================
# §package      DE_EPAGES::PayPalPPCP::Hooks::Basket
# §state        public
# §description  Additional hook functions for Basket class
#========================================================================================
package DE_EPAGES::PayPalPPCP::Hooks::Basket;

use strict;
use DE_EPAGES::PayPalPPCP::Hooks::OrderProcess;

#========================================================================================
# §function     OnUI_ViewClassicalCheckoutOffer
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       OnUI_ViewClassicalCheckoutOffer( $hParams );
#----------------------------------------------------------------------------------------
# §description  Triggered by hook UI_ViewClassicalCheckoutOffer.
#
#               Base SF/Mobile, basket, check and order page
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Basket - basket object</li>
#                   <li>Servlet - servlet</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnUI_ViewClassicalCheckoutOffer {
    my ($hParams) = @_;

    #-- This variable is of our own devising, we use it to tell our multicheckout hooks that we have been called from the classic process
    #-- This is so that we can reuse the multicheckout hook below, to save us from rewriting/refactoring the same code
    $hParams->{'PPCPClassic'} = 1;
    DE_EPAGES::PayPalPPCP::Hooks::OrderProcess::OnUI_ViewMultiCheckoutConfirmation($hParams);
}


#========================================================================================
# §function     OnUI_AfterSaveOrder
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       OnUI_AfterSaveOrder( $hParams );
#----------------------------------------------------------------------------------------
# §description  Triggered by hook UI_AfterSaveOrder.
#
#               Base SF/Mobile, basket, after change action 'Save'
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Basket - basket object</li>
#                   <li>Servlet - servlet</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnUI_AfterSaveOrder {
    my ($hParams) = @_;

    #-- This variable is of our own devising, we use it to tell our multicheckout hooks that we have been called from the classic process
    #-- This is so that we can reuse the multicheckout hook below, to save us from rewriting/refactoring the same code
    $hParams->{'PPCPClassic'} = 1;
    DE_EPAGES::PayPalPPCP::Hooks::OrderProcess::OnUI_AfterSaveMultiCheckoutPayment($hParams);
}

1;
