#========================================================================================
# package      DE_EPAGES::Product::API::ProductImageValidation
# state        public
#----------------------------------------------------------------------------------------
# description  Class for validation methods regarding product images.
#========================================================================================
package DE_EPAGES::Product::API::ProductImageValidation;

use List::MoreUtils qw ( uniq );
use DE_EPAGES::Product::API::Constants qw ( PRODUCT_IMAGES_SIZES );

use strict;



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $Product );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# input        $Product | Product | DE_EPAGES::Product::API::Object::Product
# return       $self | self | DE_EPAGES::Product::API::ProductImageValidation
#========================================================================================
sub new {
    my $class = shift;
    my ($Product) = @_;
    my $self  = bless {
        'Product' => $Product,
    }, $class;
    return $self;
}



sub getImageCount {
    my $self = shift;

    my $Product = $self->{'Product'};
    my $aPublicFiles = $Product->get('PublicFiles');

    # extract files which have a Mimetype beginning with 'image'
    my @ImageFiles = grep {lc($_->{'Mimetype'}) =~ /^image/ } @$aPublicFiles;

    # creating new array containg just the 'FileName' attribute
    my @ImageFilesFileNames = map {$_->{'FileName'}} @ImageFiles;

    # removing all image size suffixes from the file names.
    my @ImageFilesFileNamesWithoutSizeSuffix = map { $self->_removeSizeSuffix($_)} @ImageFilesFileNames;

    # remove duplicates
    my @DistinctImageFilesFileNames = uniq @ImageFilesFileNamesWithoutSizeSuffix;

    my $ImageFilesCount = @DistinctImageFilesFileNames;
    return $ImageFilesCount;
}



sub _removeSizeSuffix {
    my $self = shift;
    my ($ImageName) = @_;

    # substitute long suffixes first
    my @SizeSuffixes = sort { length($b) <=> length($a) } keys %{PRODUCT_IMAGES_SIZES()};
    for my $SizeSuffix (@SizeSuffixes) {
        $ImageName =~ s/_$SizeSuffix//g;
    }

    return $ImageName;
}

1;
