#========================================================================================
# package      DE_EPAGES::Product::t::ProductCategoriesResource
# state        public
# base         DE_EPAGES::Shop::API::TestImportXML
# description  tests the REST resource for product-categories.
#========================================================================================
package DE_EPAGES::Product::t::ProductCategoriesResource;
use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::Product::API::Resource::ProductCategories;

use Test::More;

use strict;

sub shopAlias {
    return 'TestShop';
}
sub importXMLPath {
    return 'importCategoriesProducts.xml';
}

sub test {
    my $self = shift;
    $self->testGetCategoriesLink();
    done_testing();
}

sub testGetCategoriesLink {
    my $self = shift;
    $self->shop->set({
        DomainName => 'test-domain',
        HasSSLCertificate => 1
    });
    my $Shop = $self->shop;
    my $LocaleID = 'en_GB';

    my $Resource = DE_EPAGES::Product::API::Resource::ProductCategories->new();

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('Product1');
    my $Category1  = $Shop->child('Categories')->child('MinimalCategory');
    my $Category2  = $Shop->child('Categories')->child('Shirts');

    my $SSLDomain = $Shop->get('ProtocolAndServerSSL');
    my $hData = $Resource->get( $Product, $LocaleID );

    is_deeply ($hData,
      {
          'links' => [
                       {
                         'rel' => 'self',
                         'href' => $SSLDomain.'/api/categories/?productId='.$Product->guid,
                         'title' => undef
                       },
                       {
                         'rel' => 'product',
                         'href' => $SSLDomain.'/api/products/'.$Product->guid,
                         'title' => 'MinimalProductName1'
                       },
                       {
                         'rel' => 'category',
                         'href' => $SSLDomain.'/api/categories/'.$Category1->guid,
                         'title' => 'min en'
                       },
                       {
                         'rel' => 'category',
                         'href' => $SSLDomain.'/api/categories/'.$Category2->guid,
                         'title' => 'shirts en'
                       }
                     ]
        },
               'GET category');

    return;
}

__PACKAGE__->runScript();

1;
