package DE_EPAGES::Product::t::ProductLowestPriceResource;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Test::Exception;
use Test::More tests => 5;

use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Product::API::Resource::ProductLowestPrice;

sub test {
    my $self = shift;

    $self->test_get;
    $self->test_errors;
}

sub test_get {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductLowestPrice->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('MyJacket');
    my $SubProductGUID = $Product->child('SubProducts')->child('JacketBlack4')->guid;
    my $LocaleID = 'en_GB';

    my $hData = $Resource->get( $Product, $LocaleID );


    is_deeply( $hData, {
        priceInfo => {
            quantity => {
                unit => 'piece(s)',
                amount => 1
            },
            price => {
                currency => 'GBP',
                taxType => 'GROSS',
                amount => '175',
                formatted => "from \x{a3}175.00"
            }
        },
        links => [
            {
                rel => 'product',
                href => 'https://test-domain/api/products/'.$SubProductGUID
            }
        ]
    }, 'GET MyJacket lowest price' );
}

sub test_errors {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductLowestPrice->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('MyJacket');
    my $ProductNonMaster = $ProductFolder->child('MyPetrol');
    my $ProductNoOwnPrices = $ProductFolder->child('Shirt');

    throws_ok { $Resource->get( $ProductNonMaster ) } qr/ResourceNotFound/, 'GET non-master';
    throws_ok { $Resource->get( $ProductNoOwnPrices ) } qr/ResourceNotFound/, 'GET no-own-prices';

    throws_ok { $Resource->get( $Product, 'xx_YY' ) } qr/BadRequest/, 'invalid locale';
    throws_ok { $Resource->get( $Product, 'ru_RU' ) } qr/BadRequest/, 'unsupported locale';
}

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';
    $self->shop->set({
        DomainName => 'test-domain',
        HasSSLCertificate => 1
    });

    note 'importing import.xml';
    ImportObjects('import.xml', $self->shop->parent->pathString);
}

__PACKAGE__->runScript;
