package DE_EPAGES::Product::t::ProductSlideshowResource;
use base DE_EPAGES::Core::API::Test;

use strict;
use Test::Exception;
use Test::More tests => 3;
use JSON;

use DE_EPAGES::Core::API::Object::BaseObject;
use DE_EPAGES::Product::API::Resource::ProductSlideshow;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;


sub test {
    my $self = shift;

    $self->test_get;
    $self->test_get_noslideshow;
    $self->test_get_product_not_visible;
}



sub test_get {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductSlideshow->new();

    my $ImagesSlideShow = $self->_createImageSlideShow();
    my $Product = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Product::API::Object::Product', {
            Site => _MockShop(),
            IsVisible => 1,
            UseSlideshow => 1,
            ImagesSlideShow => $ImagesSlideShow,
            WebPath => '/ProductWebPath',
            ImageInformations => {},
        }
    );

    my $aExpected = $self->_createExpectedSlideshow();
    my $aFound = _sortProducts($Resource->get($Product)->{items});
    is_deeply $aFound, $aExpected, 'get slideshow images';
}

sub test_get_noslideshow {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductSlideshow->new;
    my $Product = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Product::API::Object::Product', {
            Site => _MockShop(),
            IsVisible => 1,
            UseSlideshow => 0,
            ImagesSlideShow => [],
            GUID => '24AB755E-C277-4B43-9731-DBADEC61889D',
            ImageInformations => {},
        }
    );

    is_deeply $Resource->get($Product), {'items'=>[]}, 'get slideshow from product that has no slideshow';
}

sub test_get_product_not_visible {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductSlideshow->new;

    my $ImagesSlideShow = $self->_createImageSlideShow();
    my $Product = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Product::API::Object::Product', {
            Site => _MockShop(),
            IsVisible => 0,
            UseSlideshow => 1,
            ImagesSlideShow => $ImagesSlideShow,
            WebPath => '/ProductWebPath',
            ImageInformations => {},
        }
    );

    my $aExpected = $self->_createExpectedSlideshow();
    my $aFound = _sortProducts($Resource->get($Product)->{items});
    is_deeply $aFound, $aExpected, 'get slideshow images';
}

sub _MockShop {
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', {
            ProtocolAndServerSSL => 'https://meinshop.de',
            PathString => 'DemoShop',
            LocaleID => 'de_DE',
            UseProductZoom => 1
        }
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('siteLocale', 1, ['string']);

    return $Mockify->getMockObject();
}

sub _createImageSlideShow {
    my $self = shift;
    return [
        {
            'ImageName' => 'TestImage1',
            'Sizes' => {
                'reallybig' => 'testimg1_notsmall.jpg',
                'smaller' => 'testimg1_smaller.jpg',
                'small' => 'testimg1_small.jpg',
            },
        },
        {
            'ImageName' => 'AnotherImage',
            'Sizes' => {
                'big' => 'testimg_big.jpg',
                'small' => 'img_small.jpg',
            },
        },
    ];
}

sub _createExpectedSlideshow {
    my $self = shift;
    return [
        {
            'name' => 'TestImage1',
            'sizes' => [
                {
                    'url' => 'https://meinshop.de/ProductWebPath/testimg1_notsmall.jpg',
                    'classifier' => 'reallybig',
                    'isZoomEnabled' => JSON::true,
                    'width' => undef,
                    'height' => undef,
                    'fileSize' => undef
                },
                {
                    'url' => 'https://meinshop.de/ProductWebPath/testimg1_small.jpg',
                    'classifier' => 'small',
                    'isZoomEnabled' => JSON::true,
                    'width' => undef,
                    'height' => undef,
                    'fileSize' => undef
                },
                {
                    'url' => 'https://meinshop.de/ProductWebPath/testimg1_smaller.jpg',
                    'classifier' => 'smaller',
                    'isZoomEnabled' => JSON::true,
                    'width' => undef,
                    'height' => undef,
                    'fileSize' => undef
                },
           ]
        },
        {
            'name' => 'AnotherImage',
            'sizes' => [
                {
                    'url' => 'https://meinshop.de/ProductWebPath/testimg_big.jpg',
                    'classifier' => 'big',
                    'isZoomEnabled' => JSON::true,
                    'width' => undef,
                    'height' => undef,
                    'fileSize' => undef
                },
                {
                    'url' => 'https://meinshop.de/ProductWebPath/img_small.jpg',
                    'classifier' => 'small',
                    'isZoomEnabled' => JSON::true,
                    'width' => undef,
                    'height' => undef,
                    'fileSize' => undef
                },
            ]
        }
    ];
}

sub _sortProducts {
    my ($Products) = @_;
    my @SortedProducts;
    foreach my $Item (@$Products) {
        my @sizes = sort { $a->{classifier} cmp $b->{classifier} } @{$Item->{sizes}};
        delete $Item->{sizes};
        $Item->{sizes} = \@sizes;
        push @SortedProducts, $Item;
    };
    return \@SortedProducts
}

__PACKAGE__->runScript;
