#========================================================================================
# package      DE_EPAGES::Product::t::unit::Tools
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Product::API::Tools
#========================================================================================
package DE_EPAGES::Product::t::unit::Tools;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Product::API::Tools qw (
    MergeUpdatedProductsResults
    ConvertDateTimeForEpJ
);
use Test::More;

use strict;



## no critic (ProhibitMagicNumbers)



sub test {
    my $self = shift;

    $self->test_MergeUpdatedProductsResults();
    $self->test_ConvertDateTimeForEpJ();

    done_testing();
}



sub test_MergeUpdatedProductsResults {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $aUpdatedProducts1 = [
        [35, '2016-01-21 14:14:56'],
        [56, '2015-06-27 21:35:42'],
        [48, '2015-11-15 05:58:12'],
        [71, '2015-08-22 12:52:31'],
    ];
    my $aUpdatedProducts2 = [
        [11, '2014-06-14 12:12:44'],
        [56, '2015-04-27 11:42:41'],
        [35, '2016-01-23 21:15:27'],
    ];

    my @aUpdatedProducts = sort { $a->[0] <=> $b->[0] }
    @{MergeUpdatedProductsResults($aUpdatedProducts1, $aUpdatedProducts2)};

    my $hExpectedUpdatedProducts = [
        [
            '11',
            '2014-06-14 12:12:44'
        ],
        [
            '35',
            '2016-01-23 21:15:27'
        ],
        [
            '48',
            '2015-11-15 05:58:12'
        ],
        [
            '56',
            '2015-06-27 21:35:42'
        ],
        [
            '71',
            '2015-08-22 12:52:31'
        ],
    ];

    is_deeply(\@aUpdatedProducts, $hExpectedUpdatedProducts, "$SubTestName - test if merge contains no duplicates and only the newest entries");
}



sub test_ConvertDateTimeForEpJ {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Ep6DateTimeString = '2016-01-24 15:24:46';

    my $EpJDateTimeString = ConvertDateTimeForEpJ($Ep6DateTimeString);

    is($EpJDateTimeString, '2016-01-24T14:24:46Z', "$SubTestName - test if datetime string was correctly converted");
}

__PACKAGE__->runScript();
