#========================================================================================
# package      DE_EPAGES::Product::t::unit::WatchedProductsResource
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Product::API::Resource::WatchedProductsResource
#========================================================================================
package DE_EPAGES::Product::t::unit::WatchedProductsResource;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Product::API::Resource::WatchedProductsResource;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::Mockery;
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_get();

    done_testing();
}



sub test_get {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $Resource = $Mockery->get('SUT');
    my $Shop = $Mockery->get('Shop');

    my $hWatchedProducts = $Resource->get($Shop, 'de_DE');

    my $hExpectedWatchedProducts = $self->_createExpectedWatchedProducts();
    is_deeply($hWatchedProducts, $hExpectedWatchedProducts, "$SubTestName - test if expected data was returned.");

    my $aGetWatchedProductDetailsArgs1 = GetParametersFromMockifyCall($Mockery->get('WatchedProductsService'), 'getWatchedProductDetails', 0);
    is($aGetWatchedProductDetailsArgs1->[1], 'de_DE', "$SubTestName - test if getWatchedProductDetails was called with the right locale");
    is($aGetWatchedProductDetailsArgs1->[2], '12345', "$SubTestName - test if getWatchedProductDetails was called with the right objectid");
    is($aGetWatchedProductDetailsArgs1->[3], '5', "$SubTestName - test if getWatchedProductDetails was called with right number of watchers");

    my $aGetWatchedProductDetailsArgs2 = GetParametersFromMockifyCall($Mockery->get('WatchedProductsService'), 'getWatchedProductDetails', 1);
    is($aGetWatchedProductDetailsArgs2->[1], 'de_DE', "$SubTestName - test if getWatchedProductDetails was called with the right locale");
    is($aGetWatchedProductDetailsArgs2->[2], '31415', "$SubTestName - test if getWatchedProductDetails was called with the right objectid");
    is($aGetWatchedProductDetailsArgs2->[3], '3', "$SubTestName - test if getWatchedProductDetails was called with right number of watchers");
}



sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['ShopRequestContext', 'WatchedProductsService'], sub {
        my ($ShopRequestContext, $WatchedProductsService) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Product::API::Resource::WatchedProductsResource', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createShopRequestContext', $ShopRequestContext, ['object']);
        $Mockify->addMockWithReturnValue('_createWatchedProductsService', $WatchedProductsService);
        return $Mockify->getMockObject();
    });

    $Mockery->register('WatchedProductsService', ['ProductClass'], sub {
        my ($ProductClass) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Product::API::WatchedProductsService', [$ProductClass]);
        $Mockify->addMockWithReturnValueAndParameterCheck('getNumberOfWatchersPerProduct', {
           '12345' => '5',
           '31415' => '3',
        }, [{'int' => 5678}]);
        $Mockify->addMockWithReturnValueAndParameterCheck('getWatchedProductDetails', $self->_createProductDetails(), ['object', 'string', 'int', 'int']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('ShopRequestContext', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Resource::ShopRequestContext', [$Shop]);
        $Mockify->addMockWithReturnValueAndParameterCheck('validLocale', 'de_DE', [{'string' => 'de_DE'}]);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Shop', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'ID' => 5678,
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('ProductClass', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Class', {});
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}



sub _createExpectedWatchedProducts {
    my $self = shift;
    return {
        'page' => 1,
        'resultsPerPage' => 10,
        'results' => 2,
        'items' => [
            $self->_createProductDetails(),
            $self->_createProductDetails(),
        ]
    };
}



sub _createProductDetails {
    return {
        'product1' => 'test'
    };
}

__PACKAGE__->runScript();
