#========================================================================================
# package      DE_EPAGES::ShopMigration::API::Attributes::Shop_Ext
# base         DE_EPAGES::Object::API::BaseAttribute
# state        private
# description  Implements calculated attributes
#========================================================================================
package DE_EPAGES::ShopMigration::API::Attributes::Shop_Ext;
use base DE_EPAGES::Object::API::BaseAttribute;
use strict;

use DE_EPAGES::ShopMigration::API::Constants qw(
    SHOPMIGRATION_NOT_PARTICIPATING
    SHOPMIGRATION_INITIAL_ALLOW
    SHOPMIGRATION_INITIAL_IN_PROGRESS
    SHOPMIGRATION_INITAL_PREPARED
    SHOPMIGRATION_INITAL_FAILED
    SHOPMIGRATION_FINAL_ALLOW
    SHOPMIGRATION_FINAL_IN_PROGRESS
    SHOPMIGRATION_FINAL_SHOPTYPECHANGE_ALLOW
    SHOPMIGRATION_FINAL_DONE
    SHOPMIGRATION_FINAL_FAILED
    SHOPMIGRATION_ABORTED
    SHOPMIGRATION_FINAL_DONE_AND_PREP_DELETED
);

##no critic qw(ProhibitCascadingIfElse)

#========================================================================================
# function     getAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Package->getAttributes( $Object, $aNames );
# example      # never call getAttributes directly!
#               $Value = $Object->get( $AttributeName );
#----------------------------------------------------------------------------------------
# description  Implements calculated attributes
#               Used by <function DE_EPAGES::Object::Object::Object::get>
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $aNames | list of attribute names | ref.array
# return       $hValues | attribute name => value hash | ref.hash
#========================================================================================
sub getAttributes {
    my $self = shift;
    my ($Object, $aNames) = @_;
    my $hInfo = {};
    foreach my $Name (@$aNames) {
        if ($Name eq 'MigrationStatusString') {
            my $Status = $Object->get('MigrationStatus');
            my $String = 'UNKOWN_STATUS';

            if ($Status == SHOPMIGRATION_NOT_PARTICIPATING) {$String = 'NOT_PARTICIPATING';}
            elsif ($Status == SHOPMIGRATION_INITIAL_ALLOW) {$String = 'INITIAL_ALLOW';}
            elsif ($Status == SHOPMIGRATION_INITIAL_IN_PROGRESS) {$String = 'INITIAL_IN_PROGRESS';}
            elsif ($Status == SHOPMIGRATION_INITAL_PREPARED) {$String = 'INITAL_PREPARED';}
            elsif ($Status == SHOPMIGRATION_INITAL_FAILED) {$String = 'INITAL_FAILED';}
            elsif ($Status == SHOPMIGRATION_FINAL_ALLOW) {$String = 'FINAL_ALLOW';}
            elsif ($Status == SHOPMIGRATION_FINAL_IN_PROGRESS) {$String = 'FINAL_IN_PROGRESS';}
            elsif ($Status == SHOPMIGRATION_FINAL_SHOPTYPECHANGE_ALLOW) {$String = 'FINAL_SHOPTYPECHANGE_ALLOW';}
            elsif ($Status == SHOPMIGRATION_FINAL_DONE) {$String = 'FINAL_DONE';}
            elsif ($Status == SHOPMIGRATION_FINAL_FAILED) {$String = 'FINAL_FAILED';}
            elsif ($Status == SHOPMIGRATION_ABORTED) {$String = 'ABORTED';}
            elsif ($Status == SHOPMIGRATION_FINAL_DONE_AND_PREP_DELETED) {$String = 'FINAL_DONE_AND_PREP_DELETED';}

            $hInfo->{$Name} = $String;
        }
    }
    return $hInfo;
}

1;
